<?php

class CacheEAccelerator extends Cache
{
    public function init()
    {
        parent::init();

        if (!function_exists('eaccelerator_get')) {
            throw new \Exception('CacheEAccelerator requires PHP eAccelerator extension to be loaded, enabled or compiled with the "--with-eaccelerator-shared-memory" option.');
        }
    }

    protected function getValue($key)
    {
        $result = eaccelerator_get($key);

        return $result !== null ? $result : false;
    }

    protected function setValue($key, $value, $expire)
    {
        return eaccelerator_put($key, $value, $expire);
    }

    protected function addValue($key, $value, $expire)
    {
        return (null === eaccelerator_get($key)) ? $this->setValue($key, $value, $expire) : false;
    }

    protected function existsValue($key)
    {
        return (eaccelerator_get($key) !== null);
    }

    protected function deleteValue($key)
    {
        return eaccelerator_rm($key);
    }

    public function flush($groupName = false)
    {
        # удаляем просроченный кеш
        eaccelerator_gc();

        # удаляем оставшийся, по ключу
        $keys = eaccelerator_list_keys();
        foreach ($keys as $key) {
            $key = substr($key['name'], 1);
            if ($groupName !== false) {
                if (strpos($key, $groupName) === 0) {
                    eaccelerator_rm($key);
                }
            } else {
                $this->deleteValue($key);
            }
        }
    }
}