<?php namespace bff\db;

/**
 *  Класс по работе с полями: images, favorite-image, images-count
 *  Структура таблицы:
 *  `img` text               //fieldImages
 *  `imgfav` varchar(255) NOT NULL    //fieldFav
 *  `imgcnt` tinyint(1) unsigned NOT NULL default 0 //fieldCount
 *
 *  Changelog:
 * @version 1.45  (19.02.2012) мелкие правки
 * @version 1.4   (20.02.2011) добавлена работа со столбцом "imgcnt", для подсчета кол-ва прикрепленных изображений,
 *                              изменен порядок параметров ф-ции setParams
 * @version 1.3   (26.04.2010) added saveImageFileFav, removed IMAGETYPE_WBMP
 * @version 1.221 (07.03.2009) fix uploadImages: changed 'name' to 'tmp_name'
 * @version 1.22  (06.03.2009) (mod uploadImages) +image_type_to_extension
 * @version 1.21  (17.02.2009) (mod saveImageFile)
 * @version 1.2   (16.02.2009)
 */

use bff\utils\Files;

abstract class ImagesField extends \Component
{
    //название таблицы и полей
    protected $table = '';
    protected $fID = 'id';
    protected $fImages = 'img';
    protected $fFav = '';
    protected $fCount = '';

    //флаг, определяющий работу с полем 'fFav'
    protected $useFav = false;
    //флаг, определяющий работу с полем 'fCount' 
    protected $useCount = false;

    public function __construct()
    {
    }

    function setParams($sTableName, $sFieldID = 'id', $sFieldImages = 'images', $sFieldFav = false, $sFieldCount = false)
    {
        $this->table = $sTableName;
        $this->fID = $sFieldID; //id
        $this->fImages = $sFieldImages; //img, images, photos

        if ($sFieldFav !== false) {
            $this->fFav = $sFieldFav; //imgfav, images_fav, photofav
            $this->useFav = true;
        }

        if ($sFieldCount !== false) {
            $this->fCount = $sFieldCount; //imgcnt, images_cnt, photocnt
            $this->useCount = true;
        }
    }

    /**
     * Заполнение поля
     * @param integer $ID ID записи
     * @param string $sImages строка имен файлов @example: '1.jpg, 2.jpg' (CSV)
     * @param integer $nFavoriteIndex FAV индекс в массиве имен файлов
     */
    function setImages($ID, $sImages, $nFavoriteIndex = 0)
    {

        if (empty($sImages)) {
            return false;
        }

        $aImages = explode(',', $sImages);
        if (empty($aImages)) {
            return false;
        }

        $sImagesOld = $this->db->one_data("SELECT $this->fImages FROM $this->table WHERE $this->fID=$ID");

        $sQuery = "UPDATE $this->table
                  SET $this->fImages = " . $this->db->str2sql((!empty($sImagesOld) ? $sImagesOld . ',' : '') . $sImages) .
            ($this->useCount ? ", $this->fCount = $this->fCount + " . sizeof($aImages) : '') . "
                  WHERE $this->fID = $ID";
        $this->db->exec($sQuery);

        //если FAV еще не был создан
        if ($this->useFav && empty($sImagesOld)) {
            if (!isset($aImages[$nFavoriteIndex])) {
                $nFavoriteIndex = 0;
            }

            $sQuery = "UPDATE $this->table
                       SET $this->fFav = " . $this->db->str2sql($aImages[$nFavoriteIndex]) . "
                       WHERE $this->fID=$ID";
            $this->db->exec($sQuery);

            $this->saveImageFileFav($ID, $aImages[$nFavoriteIndex], 0);
        }

        return true;
    }

    /**
     * Заполнение FAV поля
     * @param integer $ID ID записи
     * @param string $sImageFavorite имя FAV файлa @example: '1.jpg'
     */
    function setImageFavorite($ID, $sImageFavorite)
    {
        if ($this->useFav && !empty($sImageFavorite)) {
            //check, if FAV in images array                                 
            $aData = $this->db->one_array("SELECT $this->fImages" . ($this->useFav ? ", $this->fFav" : '') . " FROM $this->table WHERE $this->fID=$ID");
            if (mb_strpos($aData[$this->fImages], $sImageFavorite) === false) {
                return false;
            }

            $sQuery = "UPDATE $this->table
                       SET $this->fFav = " . $this->db->str2sql($sImageFavorite) . "
                       WHERE $this->fID=$ID";
            $this->db->exec($sQuery);

            $this->saveImageFileFav($ID, $sImageFavorite, $aData[$this->fFav]);

            return true;
        }

        return false;
    }

    /**
     * Удаление одного изображения
     * @param integer $ID ID записи
     * @param string $sImage имя файлa изображения, которое необходимо удалить @example: '1.jpg'
     */
    function deleteImage($ID, $sImage)
    {
        if (!empty($sImage)) {
            return $this->deleteImagesSome($ID, array($sImage));
        }

        return false;
    }

    /**
     * Удаление некоторых изображений
     * @param integer $ID ID записи
     * @param array $aImagesDelete массив имен изображений, которые необходимо удалить
     */
    function deleteImagesSome($ID, $aImagesDelete)
    {
        if (empty($aImagesDelete)) {
            return false;
        }

        $aData = $this->db->one_array("SELECT $this->fImages" . ($this->useFav ? ", $this->fFav" : '') . " FROM $this->table WHERE $this->fID=$ID");
        $aImagesLeave = array_diff(explode(',', $aData[$this->fImages]), $aImagesDelete); //array of images to leave

        foreach ($aImagesDelete as $v) {
            $this->deleteImageFile($ID, $v, ($this->useFav ? $v == $aData[$this->fFav] : false));
        }

        if (count($aImagesLeave) == 0) {
            // удаляем все изображения
            $sqlEmpty = $this->db->str2sql('');
            $this->db->exec("UPDATE $this->table SET $this->fImages=" . $sqlEmpty .
                ($this->useFav ? ", $this->fFav=" . $sqlEmpty : '') .
                ($this->useCount ? ", $this->fCount=0" : '') .
                " WHERE $this->fID=$ID"
            );
        } else {
            //если FAV был удален, берем первый файл из оставшихся
            $sAdd2Query = '';
            if ($this->useFav && in_array($aData[$this->fFav], $aImagesLeave) === false) {
                $sNewFavFilename = current($aImagesLeave);
                $sAdd2Query .= ", $this->fFav = " . $this->db->str2sql($sNewFavFilename);
                $this->saveImageFileFav($ID, $sNewFavFilename, 0);
            }
            if ($this->useCount) {
                $sAdd2Query .= ", $this->fCount = " . count($aImagesLeave);
            }
            $this->db->exec("UPDATE $this->table SET $this->fImages=" . $this->db->str2sql(join(',', $aImagesLeave)) . " $sAdd2Query WHERE $this->fID=$ID");
        }

        return true;
    }

    /**
     * Удаление всех изображений
     * @param mixed $mID ID записи
     * @param boolean $bUpdateRows флаг, определяющий требуется ли актуализировать записи в БД
     */
    function deleteImagesAll($mID, $bUpdateRows = true)
    {
        $aData = $this->db->one_array("SELECT $this->fImages " .
            ($this->useFav ? ", $this->fFav" : '') . "
                                  FROM $this->table WHERE $this->fID=$mID"
        );

        $aImages = explode(',', $aData[$this->fImages]);
        if (empty($aImages)) {
            return true;
        }

        foreach ($aImages as $v) {
            $this->deleteImageFile($mID, $v, (!$this->useFav ? false : $v == $aData[$this->fFav]));
        }

        if ($bUpdateRows) {
            $this->db->exec("UPDATE $this->table SET $this->fImages=" . $this->db->str2sql('') .
                ($this->useFav ? ", $this->fFav=" . $this->db->str2sql('') : '') .
                ($this->useCount ? ", $this->fCount=0" : '') .
                " WHERE $this->fID=$mID"
            );
        }
    }

    /**
     * Загрузка файлов изображений, при помощи input=file
     * @param integer $ID ID записи
     * @param integer $nGenerateLettersCount кол-во букв в сгенерированном имени файла
     * @return string имена загруженных файлов @example: '1.jpg,2.jpg' (CSV)
     */
    function uploadImages($ID, $nGenerateLettersCount = 5)
    {
        $aFilenames = array();
        if (\Request::isPOST()) {
            $ID = intval($ID);

            if (empty($nGenerateLettersCount)) {
                $nGenerateLettersCount = 5;
            }

            foreach ($_FILES as $sInputName => $aFile) {
                if ($aFile['name'] != '') {
                    $pUpload = new \CUploader($sInputName);
                    $sExtension = Files::getExtension($aFile['name']);
                    $aImageSize = getimagesize($aFile['tmp_name']);
                    if ($aImageSize !== false
                        && in_array($sExtension, array('jpg', 'png', 'gif'))
                        && in_array($aImageSize[2], array(IMAGETYPE_GIF, IMAGETYPE_JPEG, IMAGETYPE_PNG))
                    ) {
                        $sFilename = \func::generator($nGenerateLettersCount);
                        $sFilenameFull = "$sFilename.$sExtension";
                        if ($this->saveImageFile($ID, $sFilenameFull, $pUpload, array(
                                'name' => $sFilename,
                                'ext'  => $sExtension
                            )
                        )
                        ) {
                            $aFilenames[] = $sFilenameFull;
                        }
                    }
                }
            }
        }

        if (count($aFilenames) > 0) {
            $sFilenames = implode(',', $aFilenames);
        }

        return (isset($sFilenames) ? $sFilenames : '');
    }

    /**
     * Загрузка файлов изображений через swfuploader
     * @param integer $ID ID записи
     * @param integer $nGenerateLettersCount кол-во букв в сгенерированном имени файла
     * @return string имена загруженных файлов @example: '1.jpg,2.jpg' (CSV)
     */
    function uploadImageSWF($ID, $nGenerateLettersCount = 5)
    {
        \CUploader::swfuploadStart();

        $sImg = $this->uploadImages($ID, $nGenerateLettersCount);
        $this->setImages($ID, $sImg);

        \CUploader::swfuploadFinish($sImg);
    }

    /**
     * Сохранение файла изображения
     * @param integer $ID ID записи
     * @param string $sFilenameFull @ref полное имя файла
     * @param \CUploader $pUpload @ref
     * @param array $aFilenameParams массив (name=> имя файла(без точки и расширения), ext=>расширение )
     */
    function saveImageFile($ID, &$sFilenameFull, &$pUpload, $aFilenameParams)
    {
        return false;
    }

    /**
     * Сохранение файла избранного изображения, в случае если требуется отдельный формат
     * @param integer $ID ID записи
     * @param string $sFilenameNew имя нового избранного файла
     * @param string $sFilenameOld имя предыдущего избранного файла
     */
    function saveImageFileFav($ID, $sFilenameNew, $sFilenameOld = '')
    {
        return false;
    }

    /**
     * Удаление файла изображения
     * @param integer $ID ID записи
     * @param string $sFilename имя файла
     * @param boolean $bFavotite удаляемый файл был избранным
     */
    function deleteImageFile($ID, $sFilename, $bFavotite = false)
    {
        return false;
    }

}