<?php

/**
 * Класс работы с файлами локализаций gettext
 * @version 0.3
 * @modified 19.feb.2014
 */

use bff\utils\Files;

class CGettextHelper
{
    /** @var \bff\base\Locale object */
    protected $locale = null;

    /** @var integer максимальное кол-во бекапов *.pot файла */
    protected $poFileMaxBackups = 3;
    /** @var CGettextPoFile object */
    protected $poFileObj = null;

    /** @var string регулярное выражение для поиска конструкций {t}{/t} в шаблонах smarty */
    protected $exprTPL = '';
    /** @var string регулярное выражение для поиска конструкций _t() в php файлах */
    protected $exprPHP = '';

    /** @var string имя pot файла */
    const msgFilename = 'msg';
    
    public function __construct()
    {
        $this->locale = \bff::locale();

        require_once PATH_CORE.'gettext/CGettextPoFile.php';
        $this->poFileObj = new CGettextPoFile();
    }
    
    public function generatePotFiles($sRootPath, $aLocales, $aExlude = array('.svn', '.idea', '/tpl_c'))
    {
        //prepare TPL parse expression
        $tLeftQ = preg_quote('{'); // smarty open tag   
        $tRightQ = preg_quote('}'); // smarty close tag  
        $tCmd = preg_quote('t'); // smarty command   
        //$this->exprTPL = "/{$tLeftQ}\s*({$tCmd})\s*([^{$tRightQ}]*){$tRightQ}([^{$tLeftQ}]*){$tLeftQ}\/\\1{$tRightQ}/";
        $this->exprTPL = "/{$tLeftQ}({$tCmd})\s([^{$tRightQ}]*)c=[\'\"]([^{$tRightQ}]*)[\'\"]\s*([^{$tRightQ}]*){$tRightQ}([^{$tLeftQ}]*){$tLeftQ}\/\\1{$tRightQ}/";
        
        //echo "/{$tLeftQ}({$tCmd})\s([^{$tRightQ}]*)c=[\'\"]([^{$tRightQ}]*)[\'\"]\s*([^{$tRightQ}]*){$tRightQ}([^{$tLeftQ}]*){$tLeftQ}\/\\1{$tRightQ}/"; exit;
        
        //prepare PHP parse expression
        $translator = '_t';
        //$this->exprPHP = '/\b'.$translator.'\(\s*(\'.*?(?<!\\\\)\'|".*?(?<!\\\\)")\s*[,\)]/s';
        //$this->exprPHP = '/\b'.$translator.'\s*\(\s*(\'.*?(?<!\\\\)\'|".*?(?<!\\\\)")\s*,\s*(\'.*?(?<!\\\\)\'|".*?(?<!\\\\)")\s*[,\)]/s';  
        $this->exprPHP = '/\b'.$translator.'\s*\(\s*\'(.*?)\'\s*,\s*[\'\"](.*?)[\'\"]\s*[,\)]/s'; // 2 params: ('context', 'text'...
        
        $files = Files::getFiles($sRootPath, '', true, true, array('tpl', 'php'), $aExlude);

        $messages = array();    
        foreach($files as $file) {
            $messages = array_merge($messages, $this->parseMessages($file, Files::getExtension($file) ) );
        }
        
        $messages = array_unique($messages);

        $updated = false;
        foreach($aLocales as $localeKey=>$localeData)
        {
            if( $this->savePoFile($messages, $this->locale->gt_Path($localeKey), true) )
                $updated = true;            
        }
        
        return $updated;          
    }
    
    public function updatePoFiles($aUploadedFiles)
    {
        foreach($aUploadedFiles as $localeKey=>$fileName)
        {
            $messages = $this->poFileObj->load($fileName);
            // сравниваем с текущим файлом, может чего потеряли, делаем бекап, обновляем
            $this->savePoFile($messages, $this->locale->gt_Path($localeKey), false);
        }
    }
    
    public function updateMoFiles($aLocales, $moCurrentID='', $moNextID='')
    {
        require_once PATH_CORE.'gettext/CGettextMoFile.php';
        $moFileObj = new CGettextMoFile(false);

        //обновляем *.mo файлы основываясь на *.pot файлах
        foreach($aLocales as $localeKey=>$localeData)
        {
            $langDir = $this->locale->gt_Path($localeKey);

            //frontend translation:

                # загружаем сообщения из текущего *.pot файла
                $messages = $this->poFileObj->load( self::getPOFilename( $langDir ), array('', 'common') );
                ksort( $messages );

                # обновляем
                $moCurrentFile = "$langDir".DIRECTORY_SEPARATOR."$moCurrentID-$localeKey.mo";
                if(is_file($moCurrentFile)) {
                    $moFileObj->save($moCurrentFile, $messages);
                    if( ! empty($moNextID)) # меняем имя *.mo файла, если было указано новое
                        rename($moCurrentFile, "$langDir".DIRECTORY_SEPARATOR."$moNextID-$localeKey.mo");
                } else { # создаем новый
                    $moFileObj->save("$langDir".DIRECTORY_SEPARATOR."$moNextID-$localeKey.mo", $messages);
                }

            /*
            //admin translation:

                # загружаем сообщения из текущего *.pot файла
                $messages = $this->poFileObj->load( self::getPOFilename( $langDir ), array('', 'admin', 'common') );
                ksort( $messages );

                # обновляем
                $moCurrentFile = "$langDir/$moCurrentID-$localeKey-admin.mo";
                if(is_file($moCurrentFile)) {
                    $moFileObj->save($moCurrentFile, $messages);
                    if( ! empty($moNextID)) # меняем имя *.mo файла, если было указано новое
                        rename($moCurrentFile, "$langDir/$moNextID-$localeKey-admin.mo");
                } else { # создаем новый
                    $moFileObj->save("$langDir/$moNextID-$localeKey-admin.mo", $messages);
                }
            */
        }
    }    
                                                                    
    protected function parseMessages($fileName, $fileExtension)
    {
        $content = @file_get_contents($fileName);
        if(empty($content)) return array();

        $messages = array(); 
        $moduleSeparator = '|';
        if($fileExtension == 'tpl')
        {      
            $n = preg_match_all($this->exprTPL, $content, $matches, PREG_SET_ORDER);
            //1 - func name, 3 - module, 4 - params, 4 - text
            if($n>0) 
            {
                $context = $this->getContextByFilepath($fileName);
                for($i=0; $i<$n; ++$i) {
                    $messages[] = $context.($matches[$i][3]!='' ? $matches[$i][3].$moduleSeparator : '').$this->fix($matches[$i][5]);
                }
            }
        } 
        elseif($fileExtension == 'php') 
        {                
            $n = preg_match_all($this->exprPHP, $content, $matches, PREG_SET_ORDER);
            //1 - module, 2 - text
            if($n>0) {
                $context = $this->getContextByFilepath($fileName); 
                for($i=0; $i<$n; ++$i) {         
                    $messages[] = $context.($matches[$i][1]!='' ? $matches[$i][1].$moduleSeparator : '').$matches[$i][2];
                }        
            }
        }
        return $messages;
    }
    
    public function getContextByFilepath($sFilepath)
    {
        if(strpos($sFilepath, 'admin') !== false ||
           strpos($sFilepath, 'adm.')  !== false) {
            return "admin\004";
        } elseif(strpos($sFilepath, '/bff/') !== FALSE) {
            return "common\004";
        }
        return ''; # frontend
    }
    
    // "fix" string - strip slashes, escape and convert new lines to \n
    protected function fix($str)
    {
        $str = stripslashes($str);
        $str = str_replace('"', '\"', $str);
        $str = str_replace("\n", '\n', $str);
        return $str;
    }
               
    protected function savePoFile($messages, $dir, $keysOnly)
    {              
        $fileName = self::getPOFilename($dir);
        
        if(is_file($fileName)) // обновляем существующий msg.POT файл
        {
            $translated = $this->poFileObj->load($fileName); 
            ksort($translated); 
            
            if($keysOnly) {
                sort($messages);
                if(array_keys($translated) == $messages) { //ничего нового
                    return false;
                }                
            } else {
                ksort($messages);
                if($translated == $messages) { //ничего нового
                    return false;
                }
            }
            
            $merged = array();
            $untranslated = array();     
            if($keysOnly) {
                foreach($messages as $key)
                {
                    if( ! empty($translated[$key]))
                        $merged[$key] = $translated[$key];
                    else
                        $untranslated[$key] = '';
                }
            } else {
                foreach($messages as $key=>$value)
                {
                    if( ! empty($translated[$key]))
                        $merged[$key] = $translated[$key];
                    else
                        $untranslated[$key] = '';
                }                 
            }
            
            ksort($merged); 
            ksort($untranslated); 

//            foreach($translated as $key=>$translation)
//            {
//                if( ! isset($merged[$key]) && !isset($untranslated[$key]))
//                    $untranslated[$key] = ( empty($translation) || $translation{0}!='@' ? '@@'.$translation.'@@' : $translation );
//            }
            $merged = array_merge($untranslated, $merged);
            $this->backupFile( self::getPOFilename($dir), $this->poFileMaxBackups );
        } else { # создаем новые
            $merged = array();
            if($keysOnly) {
                foreach($messages as $message)
                    $merged[$message] = '';   
            } else {
                foreach($messages as $key=>$msg)
                    $merged[$key] = $msg;
            }
        }
        
        ksort($merged);

        $this->poFileObj->save($fileName, $merged);
        return true;
    }
    
    protected function backupFile($file, $max)
    {
        for($i=$max; $i>0; --$i)
        {
            $rotateFile = $file.'.'.$i;
            if(is_file($rotateFile))
            {
                if($i === $max)
                    @unlink($rotateFile);
                else
                    @rename($rotateFile, $file.'.'.($i+1));
            }
        }
        if(is_file($file))
            rename($file, $file.'.1');
    }
    
    public static function getPOFilename( $localeDir, $sExtension = '.pot' )
    {
        return $localeDir . DIRECTORY_SEPARATOR . self::msgFilename . $sExtension;
    }

}