<?php

require_once 'model.php';

use bff\utils\Files;

abstract class DevModuleBase extends Module
{
    /** @var DevModelBase */
    public $model = null;
    protected $securityKey = 'cf145907d6219342039a032a3c2bc25c';

    /**
     * Создаем базовую структуру модуля
     * @param string $sTitle название модуля на англ.
     * @param string $sName название модуля на русском
     * @param string|bool $mInstallMarker маркер для CRUD-генератора или FALSE
     * @return bool true - модуль был успешно создан, false - ошибки при создании модуля
     */
    protected function createModule($sTitle, $sName, $mInstallMarker = false)
    {
        $nChmod = 0775;
        $bSystemError = true;
        $aLanguages = array('def');

        do {
            if (empty($sTitle)) {
                $this->errors->set('Укажите название модуля (на английском)');
                break;
            }

            $sModuleName = tpl::ucfirst($sTitle);
            $sModuleFileName = mb_strtolower($sModuleName);
            $sModuleDirectory = PATH_MODULES . $sModuleFileName . DS;

            if (file_exists($sModuleDirectory . $sModuleName . '.class.php')) {
                $this->errors->set('Модуль с таким названием уже существует', $bSystemError);
                break;
            }

            if (!Files::makeDir($sModuleDirectory, $nChmod)) {
                $this->errors->set(sprintf('Невозможно создать директорию "%s"', PATH_MODULES . $sModuleFileName), $bSystemError);
                break;
            }

            # create Template Directories
            if (!Files::makeDir($sModuleDirectory . 'tpl', $nChmod)) {
                $this->errors->set(sprintf('Невозможно создать директорию "%s"', $sModuleDirectory . 'tpl'), $bSystemError);
                break;
            }
            foreach ($aLanguages as $lng) {
                Files::makeDir($sModuleDirectory . 'tpl' . DS . $lng . DS, $nChmod);
            }

            # create Files
            $aFiles = array(
                # BL
                $sModuleFileName . '.bl.class.php'     => "<?php\n\nabstract class {$sModuleName}Base extends Module\n{\n    /** @var {$sModuleName}Model */\n    var \$model = null;\n    var \$securityKey = '" . md5(uniqid($sModuleName)) . "';\n\n\n}",
                # Model
                $sModuleFileName . '.model.php'        => "<?php\n\nclass {$sModuleName}Model extends Model\n{\n    /** @var {$sModuleName}Base */\n    var \$controller;\n    \n\n}",
                # Menu
                'm.' . $sModuleFileName . '.class.php' => "<?php\n\nclass M_$sModuleName\n{\n    static function declareAdminMenu(CMenu \$menu, Security \$security)\n    {\n        \$menu->assign('{$sName}', 'Список', '$sModuleFileName', 'listing', true, 1);\n        \n    }\n}",
                # Install.sql
                'install.sql'                          => '',
                # Admin
                $sModuleFileName . '.adm.class.php'    => "<?php\n\nclass $sModuleName extends {$sModuleName}Base\n{\n\n\n}",
                # Frontend
                $sModuleFileName . '.class.php'        => "<?php\n\nclass $sModuleName extends {$sModuleName}Base\n{\n\n\n}"
            );

            foreach ($aFiles as $file => $fileContent) {
                if (!Files::putFileContent($sModuleDirectory . $file, $fileContent)) {
                    $this->errors->set(sprintf('Невозможно создать файл "%s"', $file), $bSystemError);
                    break;
                }
            }

            return true;

        } while (false);

        return false;
    }
}