<?php

require_once 'model.php';

use bff\utils\Files;

abstract class SendmailModuleBase extends Module
{
    /** @var SendmailModelBase */
    public $model = null;
    protected $securityKey = 'e39b3410cf858526d5b3462db86a2ae8';

    /** @var array список шаблонов, доступных для редактирования array(key=>params, ...) */
    protected $aTemplates = array();
    /** начало макроса */
    protected $tagStart = '{';
    /** завершение макроса */
    protected $tagEnd = '}';

    /** @var string расширения шаблонов писем */
    protected $_tplExt = '.txt';
    /** @var string путь к шаблонам писем */
    protected $_tplPath = '';
    /** @var string путь к исходным шаблонам писем */
    protected $_tplPathSrc = '';

    public function init()
    {
        parent::init();

        $this->module_title = 'Работа с почтой';
        $this->_tplPath = PATH_BASE . 'files' . DS . 'mail' . DS;
        $this->_tplPathSrc = $this->_tplPath . 'src' . DS;
    }

    /**
     * @return Sendmail
     */
    public static function i()
    {
        return bff::module('sendmail');
    }

    /**
     * @return SendmailModel
     */
    public static function model()
    {
        return bff::model('sendmail');
    }

    /**
     * Получение шаблона письма
     * @param string $sTemplateKey ключ шаблона
     * @param array $aTplVars набор (макрос=>значение, ...), для автозамены в шаблоне
     * @param string $lng ключ языка
     * @return array (body=>string, subject=>string)
     */
    public function getMailTemplate($sTemplateKey, $aTplVars = false, $lng = LNG)
    {
        $aTemplateData = $this->getMailTemplateFromFile($sTemplateKey);

        if ($aTplVars !== false) {
            $aTplVars = array_merge($aTplVars, array(
                'siteurl' => SITEURL,
                'site.title' => config::sys('site.title', ''),
                'site.host' => config::sys('site.host', ''),
            ));
            $aReplace = array();
            foreach ($aTplVars as $key => $value) {
                $aReplace[$this->tagStart . $key . $this->tagEnd] = $value;
            }

            $aTemplateData['body'] = strtr(nl2br($aTemplateData['body'][$lng]), $aReplace);
            $aTemplateData['subject'] = strtr($aTemplateData['subject'][$lng], $aReplace);
        }

        return $aTemplateData;
    }

    /**
     * Получение шаблона письма из файла
     * @param string $sTemplateKey ключ шаблона
     * @return array (body=>string, subject=>string)
     */
    public function getMailTemplateFromFile($sTemplateKey)
    {
        static $cache = array();

        if (!empty($cache[$sTemplateKey])) {
            return $cache[$sTemplateKey];
        }

        $sTemplatePath = $this->_tplPath . $sTemplateKey . $this->_tplExt;

        # получаем шаблон письма
        if (file_exists($sTemplatePath)) {
            $sContent = Files::getFileContent($sTemplatePath);
        } else {
            $sContent = '';
        }
        if (!$sContent) {
            # нигде не нашли, восстанавливаем
            if ($this->restoreMailTemplateFile($sTemplateKey) !== false) {
                $sContent = Files::getFileContent($sTemplatePath);
            }
        }

        $sContentUn = (!empty($sContent) ? @unserialize($sContent) : false);
        if (is_array($sContentUn)) # un-сериализация прошла успешно
        {
            $sContent = $sContentUn;
        }

        # сохраняем в кеш и возвращаем шаблон
        if (!is_array($sContent)) {
            $sContent = array('body' => array(), 'subject' => array());
            foreach ($this->locale->getLanguages() as $lng) {
                $sContent['body'][$lng] = $sContent['subject'][$lng] = '';
            }
        } else {
            foreach ($this->locale->getLanguages() as $lng) {
                foreach ($sContent as $k => $v) {
                    if (!isset($sContent[$k][$lng])) {
                        $sContent[$k][$lng] = '';
                    }
                }
            }
        }

        return ($cache[$sTemplateKey] = $sContent);
    }

    /**
     * Восстановление шаблона письма из файла (доступно главному админу)
     * @param string $sTemplateKey ключ шаблона
     * @param int(octal) $chmod chmod
     */
    public function restoreMailTemplateFile($sTemplateKey, $chmod = 0666)
    {
        $sFilename = $sTemplateKey . $this->_tplExt;
        if (file_exists($this->_tplPathSrc . $sFilename) && is_dir($this->_tplPath)) {
            $bResult = copy($this->_tplPathSrc . $sFilename, $this->_tplPath . $sFilename);
            if ($chmod !== false) {
                @chmod($this->_tplPath . $sFilename, $chmod);
            }

            return $bResult;
        }

        return false;
    }

    /**
     * Сохранение шаблона письма в файл
     * @param string $sTemplateKey ключ шаблона
     * @param array $aTemplateData (body=>string, subject=>string) данные шаблона
     */
    public function saveMailTemplateToFile($sTemplateKey, $aTemplateData)
    {
        return Files::putFileContent($this->_tplPath . $sTemplateKey . $this->_tplExt, serialize($aTemplateData));
    }

    /**
     * Отправка письма
     * @param string $to email получателя
     * @param string $subject тема письма
     * @param string $body текст письма
     * @param string $from email отправителя, если пустая строка - mail.noreply(config/sys)
     * @param string $formName название отправителя, если пустая строка - mail.fromname (config/sys)
     * @return bool
     */
    public function sendMail($to, $subject, $body, $from = '', $formName = '')
    {
        $mailer = new CMail();

        if (!empty($from)) {
            $mailer->From = $from;
        }

        if (!empty($formName)) {
            $mailer->FromName = $formName;
        }

        $mailer->Subject = $subject;
        $mailer->MsgHTML($body);
        $mailer->AddAddress($to);

        $result = $mailer->Send();

        return $result;
    }

    /**
     * Обработка копирования данных локализации
     */
    public function onLocaleDataCopy($from, $to)
    {
        # шаблоны писем
        foreach ($this->aTemplates as $key => $v) {
            $templateData = $this->getMailTemplateFromFile($key);
            foreach ($templateData as &$vv) {
                if (isset($vv[$from])) {
                    $vv[$to] = $vv[$from];
                }
            }
            unset($vv);
            $this->saveMailTemplateToFile($key, $templateData);
        }
    }
}