<?php

require_once 'model.php';

abstract class SiteModuleBase extends Module
{
    /** @var SiteModelBase */
    public $model = null;
    protected $securityKey = 'bc7ce37bb936e4f217b7ae2a940f5b3e';
    # Pages
    static $pagesPath = '';
    static $pagesExtension = '.html';

    public function init()
    {
        parent::init();
        $this->module_title = 'Настройки сайта';
        static::$pagesPath = PATH_PUBLIC . 'files' . DS . 'pages' . DS;
    }

    /**
     * @return Site
     */
    public static function i()
    {
        return bff::module('site');
    }

    /**
     * @return SiteModel
     */
    public static function model()
    {
        return bff::model('site');
    }

    public function getCounters()
    {
        return $this->model->countersView();
    }

    /**
     * Получаем данные об основной валюте
     * @param string|boolean $mValueKey ключ требуемых данных, false - все данные
     * @return mixed
     */
    public static function currencyDefault($mValueKey = 'title_short')
    {
        if ($mValueKey == 'id') {
            return config::sys('currency.default');
        }

        return static::currencyData(0, $mValueKey);
    }

    /**
     * Получаем данные о необходимой валюте
     * @param int $nCurrencyID ID валюты, если 0 - возвращаем данные об основной валюте
     * @param string|boolean $mValueKey ключ требуемых данных, false - все данные
     * @return mixed
     */
    public static function currencyData($nCurrencyID = 0, $mValueKey = false)
    {
        $aData = static::model()->currencyData(false);
        if (empty($nCurrencyID)) {
            $nCurrencyID = config::sys('currency.default');
        }
        if (!isset($aData[$nCurrencyID])) {
            return false;
        }

        return ($mValueKey !== false && isset($aData[$nCurrencyID][$mValueKey]) ? $aData[$nCurrencyID][$mValueKey] : $aData[$nCurrencyID]);
    }

    /**
     * Конвертация цены в требуемую валюту, по-курсу
     * @param integer $nPrice цена
     * @param integer $nFromID ID валюты, в которой указана цена
     * @param integer $nToID ID валюты, в которую необходимо конвертировать цену
     * @return float
     */
    public static function currencyPriceConvert($nPrice, $nFromID, $nToID = 0)
    {
        if ($nFromID == $nToID # цена уже указана в требуемой валюте
            || $nPrice <= 0 # цена <= 0
            || !($nDefaultID = config::sys('currency.default'))
        ) { # ID валюты по-умолчанию указан некорректно
            return $nPrice;
        }

        # корректируем ID валют
        if (!$nFromID) {
            $nFromID = $nDefaultID;
        }
        if (!$nToID) {
            $nToID = $nDefaultID;
        }
        if ($nFromID == $nToID) {
            return $nPrice;
        }

        # конвертируем в основную валюту по курсу
        $nPriceDefault = ($nFromID == $nDefaultID ? $nPrice : round($nPrice * floatval(static::currencyData($nFromID, 'rate'))));
        if ($nToID == $nDefaultID) {
            return $nPriceDefault;
        } else {
            # конвертируем в требуемую валюту
            $nRate = floatval(static::currencyData($nToID, 'rate'));

            return ($nRate > 0 ? $nPriceDefault / $nRate : $nPriceDefault);
        }
    }

    /**
     * Конвертация цены в основную валюту, по-курсу
     * @param integer $nPrice цена
     * @param integer $nPriceCurrencyID ID валюты, в которой указана цена
     * @return float
     */
    public static function currencyPriceConvertToDefault($nPrice, $nPriceCurrencyID)
    {
        return static::currencyPriceConvert($nPrice, $nPriceCurrencyID, 0);
    }

    /**
     * Защита от спама (частой отправки сообщений / выполнения действий)
     * @param string $key ключ выполняемого действия
     * @param integer $timeout допустимая частота выполнения действия, в секундах
     * @param boolean $setError устанавливать ошибку
     * @return boolean true - частота отправки превышает допустимый лимит, false - все ок
     */
    public function preventSpam($key, $timeout = 20, $setError = true)
    {
        $timeout = intval($timeout);
        if ($timeout <= 0) {
            return false;
        }

        $last = $this->model->requestGet($key, User::id(), Request::remoteAddress());
        if ($last > 0 && ((BFF_NOW - $last) < $timeout)) {
            if ($setError) {
                if ($timeout <= 70) {
                    $this->errors->set(_t('', 'Повторите попытку через одну минуту'));
                } else {
                    $this->errors->set(_t('', 'Повторите попытку через несколько минут'));
                }
            }

            return true;
        } else {
            $this->model->requestSet($key, User::id(), Request::remoteAddress());
        }

        return false;
    }

    /**
     * Обработка копирования данных локализации
     */
    public function onLocaleDataCopy($from, $to)
    {
        # настройки сайта
        $configUpdate = array();
        $fromPostfixLen = mb_strlen('_' . $from);
        foreach (config::$data as $k => $v) {
            if (mb_strrpos($k, '_' . $from) === mb_strlen($k) - $fromPostfixLen) {
                $configUpdate[mb_substr($k, 0, -$fromPostfixLen) . '_' . $to] = $v;
            }
        }
        config::save($configUpdate);
    }
}