-- --------------------------------------------------------
-- Таблица настроек

DROP TABLE IF EXISTS "bff_config";
CREATE TABLE "bff_config" (
    config_name varchar(255) NOT NULL default '',
    config_value text,
    is_dynamic smallint NOT NULL default 0,
    CONSTRAINT bff_config_pkey PRIMARY KEY (config_name)
);
CREATE INDEX "bff_config_dynamic_idx" ON "bff_config" (is_dynamic);

-- --------------------------------------------------------
-- Запросы к сайту (для защиты от спама)

DROP TABLE IF EXISTS "bff_site_requests";
CREATE TABLE "bff_site_requests" (
  user_action varchar(100) NOT NULL default '',
  user_id integer NOT NULL default 0, -- ID пользователя
  user_ip varchar(15) NOT NULL, -- IP адрес пользователя
  created timestamp with time zone
);
CREATE INDEX "bff_site_requests_user_idx" ON "bff_site_requests" (user_action, user_id);
CREATE INDEX "bff_site_requests_ip_idx" ON "bff_site_requests" (user_action, user_ip);

-- --------------------------------------------------------
-- Таблица счетчиков

DROP TABLE IF EXISTS "bff_counters";
CREATE TABLE "bff_counters" (
  id serial,
  title varchar(200) NOT NULL default '',
  code text,
  enabled int2 NOT NULL default 0,
  created timestamp with time zone,
  num integer NOT NULL default 0,
  CONSTRAINT bff_counters_pkey PRIMARY KEY (id)
);

-- --------------------------------------------------------
-- Таблицы валют

DROP TABLE IF EXISTS "bff_currencies";
CREATE TABLE "bff_currencies" (
  id serial NOT NULL,
  keyword varchar(100) NOT NULL default '',
  enabled int2 NOT NULL default 1,
  rate numeric(7,4) NOT NULL default 0,
  num integer NOT NULL default 0,
  CONSTRAINT bff_currencies_pkey PRIMARY KEY (id)
);

DROP TABLE IF EXISTS "bff_currencies_lang";
CREATE TABLE "bff_currencies_lang" (
  id integer NOT NULL,
  lang character(2) NOT NULL default '',
  title varchar(100) NOT NULL default '',
  title_short varchar(100) NOT NULL default '',
  title_decl varchar(100) NOT NULL default '',
  CONSTRAINT bff_currencies_lang_unique UNIQUE (id, lang)
);

INSERT INTO "bff_currencies" (id, keyword, enabled, rate, num) VALUES
(1, 'rub', 1, 1, 1),
(2, 'uah', 1, 1, 2),
(3, 'usd', 1, 1, 3),
(4, 'eur', 1, 1, 4);
ALTER SEQUENCE "bff_currencies_id_seq" RESTART WITH 4;

INSERT INTO "bff_currencies_lang" (id, lang, title, title_short, title_decl) VALUES
(1, 'ru', 'Рубли', 'руб.', 'рубль;рубля;рублей'),
(2, 'ru', 'Гривна', 'грн.', 'гривна;гривны;гривен'),
(3, 'ru', 'Доллары', '$', 'доллар;доллара;долларов'),
(4, 'ru', 'Евро', 'евр.', 'евро;евро;евро');