<?php

abstract class SvcModule extends SvcModuleBase
{
    /**
     * Активируем услугу/пакет услуг или формируем счет для оплаты
     * @param string $sModuleName название модуля, в котором выполняется активация услуги
     * @param integer $nSvcID ID услуги / пакета услуг
     * @param integer $nItemID ID записи
     * @param integer $nPaySystem ID системы оплаты Bills::PS_
     * @param string $sPaySystemWay способ оплаты в системе оплаты
     * @param integer|boolean $nSvcPrice стоимость услуги / пакета услуг или FALSE (получить из настроек)
     * @return array
     */
    function activateOrPay($sModuleName, $nSvcID, $nItemID, $nPaySystem, $sPaySystemWay, $nSvcPrice = false)
    {
        $aResponse = array('activated' => false, 'pay' => false);

        $oModule = bff::module($sModuleName);
        $nUserID = $this->security->getUserID();
        do {
            if (!$oModule instanceof IModuleWithSvc) {
                bff::log('Модуль "' . $sModuleName . '" должен реализовать интерфейс "IModuleWithSvc"');
                $this->errors->reloadPage();
                break;
            }

            $aSvc = $this->svc()->model->svcData($nSvcID);
            if (empty($aSvc) || $aSvc['module'] != $sModuleName) {
                $this->errors->reloadPage();
                break;
            }
            if ($nSvcPrice === false) {
                $nSvcPrice = $aSvc['price'];
            }

            $nUserBalance = $this->security->getUserBalance();
            if ($nUserBalance >= $nSvcPrice) {
                $aResponse['activated'] = $this->activate($sModuleName, $nSvcID, $aSvc, $nItemID, $nUserID, $nSvcPrice);
            } else {
                # создаем счет для оплаты
                $nBillID = $this->bills()->createBill_InPay($nUserID, $nUserBalance, $nSvcPrice, $nSvcPrice, Site::currencyDefault('id'),
                    Bills::STATUS_WAITING,
                    $nPaySystem, $sPaySystemWay,
                    _t('bills', 'Пополнение счета'),
                    $nSvcID, true, # помечаем необходимость активации сразу после оплаты
                    $nItemID
                );
                if (!$nBillID) {
                    $this->errors->set(_t('bills', 'Ошибка создания счета'));
                    break;
                }
                $aResponse['pay'] = true;
                $aResponse['form'] = $this->bills()->buildPayRequestForm($nPaySystem, $sPaySystemWay, $nBillID, $nSvcPrice);
            }
        } while (false);

        return $aResponse;
    }
}