<?php

/**
 * Класс синхронизации с форумом
 * @version 0.11
 * @modified 11.jan.2014
 */

class UsersForum extends Component
{
    protected function dbPrefix()
    {
        return config::sys('forum.db.prefix');
    }

    /**
     * Выполняем подключение к базе форума
     * @return \bff\db\Database | bool
     */
    protected function dbConnection()
    {
        static $db;
        if (!isset($db)) {
            try {
                $db = bff::DI('database_factory');
                $db->connectionConfig('forum.db');
                $db->connect();
            } catch (\Exception $e) {
                bff::log($e->getMessage());
                $db = false;

                return false;
            }
        }

        return $db;
    }

    /**
     * Обрабатываем событие смены логина пользователем
     * @param integer $nUserID ID пользователя
     * @param string $sLoginNew новый логин
     * @param string $sLoginOld старый логин
     * @apram mixed ID текущей сессии пользователя ($nUserID)
     * @return bool true - логин успешно изменен, false - ошибка смены логина
     */
    public function onUserLoginChanged($nUserID, $sLoginNew, $sLoginOld, $mSessionID = null)
    {
        # обновляем данные в сессии (для того, чтобы их подхватил форум)
        if (!bff::adminPanel()) {
            # в текущей сессии
            $this->security->updateUserInfo(array('login' => $sLoginNew));
        } else {
            # в сессии пользователя $nUserID
            $this->security->impersonalizeSession($mSessionID, array('login' => $sLoginNew), false, false);
        }

        switch (config::sys('forum.type')) {
            case 'phpbb3':
            {
                $dbForum = $this->dbConnection();
                if (empty($dbForum)) {
                    break;
                }

                # заменяем логин(username) на новый:
                $res = $dbForum->exec('UPDATE ' . $this->dbPrefix() . 'users
                         SET username = :new, username_clean = :new
                         WHERE username = :old',
                    array(':new' => $sLoginNew, ':old' => $sLoginOld)
                );

                return !empty($res);
            }
            break;
        }

        return false;
    }

}