<?php namespace bff\utils;

/**
 * Класс вспомогательных методов обработки текста
 * @version 0.31
 * @modified 2.dec.2013
 */

class TextParser
{
    public function __construct()
    {
    }

    /**
     * Инициализация компонента Jevix
     * @return \Jevix
     */
    public function jevix()
    {
        static $i;
        if (!isset($i)) {
            require_once(PATH_CORE . 'external/jevix/jevix.class.php');
            $i = new \Jevix();
        }

        return $i;
    }

    /**
     * Парсинг текста с помощью Jevix
     * @param string $sText текст
     * @param array $aErrors @ref массив ошибок (заполняется Jevix парсером)
     * @return string обработанный текст
     */
    public function JevixParser($sText, &$aErrors = array())
    {
        $sResult = $this->jevix()->parse($sText, $aErrors);

        return $sResult;
    }

    /**
     * Парсинг текста на предмет видео
     * @param string $sText текст
     * @return string
     */
    public function VideoParser($sText)
    {
        # youtube.com
        $sText = preg_replace('/<video>http:\/\/(?:www\.|)youtube\.com\/watch\?v=([a-zA-Z0-9_\-]+)<\/video>/Ui', '<object width="425" height="344"><param name="movie" value="http://www.youtube.com/v/$1&hl=en"></param><param name="wmode" value="opaque"></param><embed src="http://www.youtube.com/v/$1&hl=en" type="application/x-shockwave-flash" wmode="opaque" width="425" height="344"></embed></object>', $sText);
        # rutube.ru
        $sText = preg_replace('/<video>http:\/\/(?:www\.|)rutube.ru\/tracks\/\d+.html\?v=([a-zA-Z0-9_\-]+)<\/video>/Ui', '<object width="470" height="353"><param name="movie" value="http://video.rutube.ru/$1"></param><param name="wmode" value="opaque"></param><param name="allowFullScreen" value="true"></param><param name="flashVars" value="uid=662118"></PARAM><EMBED src="http://video.rutube.ru/$1" type="application/x-shockwave-flash" wmode="opaque" width="470" height="353" allowFullScreen="true" flashVars="uid=662118"></embed></object>', $sText);

        return $sText;
    }

    /**
     * Заменяет все вхождения короткого тега <param/> на длиную версию <param></param>
     * Заменяет все вхождения короткого тега <embed/> на длиную версию <embed></embed>
     * @param string $sText текст
     */
    protected function FlashParamParser($sText)
    {
        if (preg_match_all("@(<\s*param\s*name\s*=\s*\".*\"\s*value\s*=\s*\".*\")\s*/?\s*>(?!</param>)@Ui", $sText, $aMatch)) {
            foreach ($aMatch[1] as $key => $str) {
                $str_new = $str . '></param>';
                $sText = str_replace($aMatch[0][$key], $str_new, $sText);
            }
        }
        if (preg_match_all("@(<\s*embed\s*.*)\s*/?\s*>(?!</embed>)@Ui", $sText, $aMatch)) {
            foreach ($aMatch[1] as $key => $str) {
                $str_new = $str . '></embed>';
                $sText = str_replace($aMatch[0][$key], $str_new, $sText);
            }
        }
        /**
         * Удаляем все <param name="wmode" value="*"></param>
         */
        if (preg_match_all("@(<param\s.*name=\"wmode\".*>\s*</param>)@Ui", $sText, $aMatch)) {
            foreach ($aMatch[1] as $key => $str) {
                $sText = str_replace($aMatch[0][$key], '', $sText);
            }
        }
        /**
         * А теперь после <object> добавляем <param name="wmode" value="opaque"></param>
         * Решение не фантан, но главное работает :)
         */
        if (preg_match_all("@(<object\s.*>)@Ui", $sText, $aMatch)) {
            foreach ($aMatch[1] as $key => $str) {
                $sText = str_replace($aMatch[0][$key], $aMatch[0][$key] . '<param name="wmode" value="opaque"></param>', $sText);
            }
        }

        return $sText;
    }

    /**
     * Делает ссылки не видимыми для поисковиков
     * @param string $sText текст
     * @return string
     */
    public function MakeUrlNoIndex($sText)
    {
        return preg_replace("/(<a .*>.*<\/a>)/Ui", "<noindex>$1</noindex>", $sText);
    }

    /**
     * Парсит текст комментария (без HTML тегов)
     * @param string $sText
     * @param boolean $bActivateLinks активировать ссылки
     * @return string
     */
    public function parseCommentPlain($sMessage, $bActivateLinks = true)
    {
        $sMessage = preg_replace("/(\<script)(.*?)(script>)/si", '', $sMessage);
        $sMessage = htmlspecialchars($sMessage);
        $sMessage = preg_replace("/(\<)(.*?)(--\>)/mi", '' . nl2br("\\2") . '', $sMessage);
        $sMessage = nl2br($sMessage);
        if ($bActivateLinks) {
            $oParser = new LinksParser();
            $sMessage = $oParser->parse($sMessage);
        }

        return $sMessage;
    }

    /**
     * Парсинг текста комментария (с HTML тегами)
     * @param string $sText текст
     */
    public function parseCommentHTML($sText)
    {
        # Конфигурирует типограф
        $j = $this->jevix();

        # Разрешённые теги
        $j->cfgAllowTags(array(
                'cut',
                'a',
                'img',
                'i',
                'b',
                'u',
                's',
                'video',
                'em',
                'strong',
                'nobr',
                'li',
                'ol',
                'ul',
                'sup',
                'abbr',
                'sub',
                'acronym',
                'h4',
                'h5',
                'h6',
                'br',
                'hr',
                'pre',
                'code',
                'object',
                'param',
                'embed',
                'blockquote'
            )
        );
        # Коротие теги типа
        $j->cfgSetTagShort(array('br', 'img', 'hr', 'cut'));
        # Преформатированные теги
        $j->cfgSetTagPreformatted(array('pre', 'code', 'video'));
        # Разрешённые параметры тегов
        $j->cfgAllowTagParams('img', array(
                'src',
                'alt'    => '#text',
                'title',
                'align'  => array('right', 'left', 'center'),
                'width'  => '#int',
                'height' => '#int',
                'hspace' => '#int',
                'vspace' => '#int'
            )
        );
        $j->cfgAllowTagParams('a', array('href', 'title', 'rel'));
        $j->cfgAllowTagParams('cut', array('name'));
        $j->cfgAllowTagParams('object', array('width' => '#int', 'height' => '#int', 'data' => '#link'));
        $j->cfgAllowTagParams('param', array('name' => '#text', 'value' => '#text'));
        $j->cfgAllowTagParams('embed', array(
                'src'               => '#image',
                'type'              => '#text',
                'allowscriptaccess' => '#text',
                'allowfullscreen'   => '#text',
                'width'             => '#int',
                'height'            => '#int',
                'flashvars'         => '#text',
                'wmode'             => '#text'
            )
        );
        # Параметры тегов являющиеся обязательными
        $j->cfgSetTagParamsRequired('img', 'src');
        $j->cfgSetTagParamsRequired('a', 'href');
        # Теги которые необходимо вырезать из текста вместе с контентом
        $j->cfgSetTagCutWithContent(array('script', 'iframe', 'style'));
        # Вложенные теги
        $j->cfgSetTagChilds('ul', array('li'), false, true);
        $j->cfgSetTagChilds('ol', array('li'), false, true);
        $j->cfgSetTagChilds('object', 'param', false, true);
        $j->cfgSetTagChilds('object', 'embed', false, false);
        # Если нужно оставлять пустые не короткие теги
        $j->cfgSetTagIsEmpty(array('param', 'embed'));
        # Теги с обязательными параметрами
        $j->cfgSetTagParamsAutoAdd('embed', array(array('name' => 'wmode', 'value' => 'opaque', 'rewrite' => true)));
        # Отключение авто-добавления <br>
        $j->cfgSetAutoBrMode(false);
        # Автозамена
        $j->cfgSetAutoReplace(array('+/-', '(c)', '(r)', '(C)', '(R)'), array('±', '©', '®', '©', '®'));
        $j->cfgSetXHTMLMode(false);
        $j->cfgSetTagNoTypography('code');
        $j->cfgSetTagNoTypography('video');

        $sResult = $this->FlashParamParser($sText);
        $sResult = $this->JevixParser($sResult);
        $sResult = $this->VideoParser($sResult);

        return $sResult;
    }

    /**
     * Парсинг wysiwyg текста
     * Метод используется компонентом {bff\db\Publicator}
     * @param string $sText текст
     * @param array $aParams доп. настройки:
     *   boolean 'scripts' - разрешать вставку script тегов
     *   boolean 'iframes' - разрешать вставку iframe тегов
     */
    public function parseWysiwygText($sText, $aParams = array())
    {
        static $configured;

        if (!isset($configured)) {
            $configured = true;
            $j = $this->jevix();

            # 1. Устанавливаем разрешённые теги. (Все не разрешенные теги считаются запрещенными.)
            $j->cfgAllowTags(array(
                    'a',
                    'img',
                    'i',
                    'b',
                    'u',
                    'em',
                    'strong',
                    'nobr',
                    'li',
                    'ol',
                    'ul',
                    'sub',
                    'sup',
                    'abbr',
                    'pre',
                    'acronym',
                    'h1',
                    'h2',
                    'h3',
                    'h4',
                    'h5',
                    'h6',
                    'adabracut',
                    'br',
                    'hr',
                    'p',
                    'span',
                    'div',
                    'code',
                    'object',
                    'param',
                    'embed',
                    'blockquote'
                )
            );

            # 2. Устанавливаем коротие теги. (не имеющие закрывающего тега)
            $j->cfgSetTagShort(array('br', 'img', 'hr'));

            # 3. Устанавливаем преформатированные теги. (в них все будет заменятся на HTML сущности)
            $j->cfgSetTagPreformatted(array('pre'));

            # 4. Устанавливаем теги, которые необходимо вырезать из текста вместе с контентом.
            if (!empty($aParams['scripts'])) {
                $j->cfgAllowTags(array('script'));
                $j->cfgSetTagIsEmpty(array('script'));
                $j->cfgAllowTagParams('script', array('src', 'type', 'charset'));
            } else {
                $j->cfgSetTagCutWithContent(array('script'));
            }
            if (!empty($aParams['iframes'])) {
                $j->cfgAllowTags(array('iframe'));
                $j->cfgSetTagIsEmpty(array('iframe'));
                $j->cfgAllowTagParams('iframe', array(
                        'align',
                        'src',
                        'frameborder',
                        'height',
                        'width',
                        'scrolling',
                        'name',
                        'marginwidth',
                        'marginheight'
                    )
                );
            } else {
                $j->cfgSetTagCutWithContent(array('iframe'));
            }
            $j->cfgSetTagCutWithContent(array('style'));

            # 5. Устанавливаем разрешённые параметры тегов. Также можно устанавливать допустимые значения этих параметров.
            $j->cfgAllowTagParams('a', array('title', 'href'));
            $j->cfgAllowTagParams('img', array(
                    'src',
                    'alt'    => '#text',
                    'title',
                    'align'  => array('right', 'left', 'center'),
                    'width'  => '#int',
                    'height' => '#int'
                )
            );
            $j->cfgAllowTagParams('span', array('style', 'align', 'class'));
            $j->cfgAllowTagParams('div', array('style', 'align', 'class'));

            # 6. Устанавливаем параметры тегов являющиеся обязяательными. Без них вырезает тег оставляя содержимое.
            $j->cfgSetTagParamsRequired('img', 'src');
            $j->cfgSetTagParamsRequired('a', 'href');

            # 7. Устанавливаем теги которые может содержать тег контейнер
            //    cfgSetTagChilds($tag, $childs, $isContainerOnly, $isChildOnly)
            //       $isContainerOnly : тег является только контейнером для других тегов и не может содержать текст (по умолчанию false)
            //       $isChildOnly : вложенные теги не могут присутствовать нигде кроме указанного тега (по умолчанию false)
            $j->cfgSetTagChilds('ul', 'li', true, false);
            $j->cfgSetTagChilds('ol', 'li', true, false);

            # 8. Устанавливаем атрибуты тегов, которые будут добавлятся автоматически
            $j->cfgSetTagParamDefault('a', 'rel', null, true);
            //$j->cfgSetTagParamsAutoAdd('a', array('rel' => 'nofollow'));
            //$j->cfgSetTagParamsAutoAdd('a', array('name'=>'rel', 'value' => 'nofollow', 'rewrite' => true));

            $j->cfgSetTagParamDefault('img', 'width', '565px');

            # 9. Устанавливаем автозамену
            $j->cfgSetAutoReplace(array('+/-', '(c)', '(r)'), array('±', '©', '®'));

            # 10. Включаем или выключаем режим XHTML. (по умолчанию включен)
            $j->cfgSetXHTMLMode(true);

            # 11. Включаем или выключаем режим замены переноса строк на тег <br/>. (по умолчанию включен)
            $j->cfgSetAutoBrMode(false);

            # 12. Включаем или выключаем режим автоматического определения ссылок. (по умолчанию включен)
            $j->cfgSetAutoLinkMode(true);

            # 13. Отключаем типографирование в определенном теге
            $j->cfgSetTagNoTypography('code');
        }

        $sText = str_replace('&nbsp;', ' ', $sText);

        return $this->JevixParser($sText, $aErrors);
    }

    /**
     * Простой метод корректировки неправильной раскладки клавиатуры
     * @param string $string строка, требующая корректировки
     * @param string $from раскладка в которой предположительно набирался текст
     * @param string $to раскладка в которую необходимо конвертировать
     * @return string
     */
    public function correctKeyboardLayout($string, $from = 'en', $to = 'ru')
    {
        static $data = array(
            'en' => array(
                'q',
                'w',
                'e',
                'r',
                't',
                'y',
                'u',
                'i',
                'o',
                'p',
                '[',
                ']',
                "\\",
                'a',
                's',
                'd',
                'f',
                'g',
                'h',
                'j',
                'k',
                'l',
                ';',
                "'",
                'z',
                'x',
                'c',
                'v',
                'b',
                'n',
                'm',
                ',',
                '.'
            ),
            'ru' => array(
                'й',
                'ц',
                'у',
                'к',
                'е',
                'н',
                'г',
                'ш',
                'щ',
                'з',
                'х',
                'ъ',
                'ё',
                'ф',
                'ы',
                'в',
                'а',
                'п',
                'р',
                'о',
                'л',
                'д',
                'ж',
                'э',
                'я',
                'ч',
                'с',
                'м',
                'и',
                'т',
                'ь',
                'б',
                'ю'
            ),
            'ua' => array(
                'й',
                'ц',
                'у',
                'к',
                'е',
                'н',
                'г',
                'ш',
                'щ',
                'з',
                'х',
                'ї',
                'ґ',
                'ф',
                'и',
                'в',
                'а',
                'п',
                'р',
                'о',
                'л',
                'д',
                'ж',
                'є',
                'я',
                'ч',
                'с',
                'м',
                'і',
                'т',
                'ь',
                'б',
                'ю'
            ),
        );
        if (!isset($data[$from]) || !isset($data[$to])) {
            return $string;
        }

        return preg_replace($data[$from], $data[$to], mb_strtolower($string));
    }

}