
DROP TABLE IF EXISTS `bff_blog_categories_lang`;
DROP TABLE IF EXISTS `bff_blog_categories`;
DROP TABLE IF EXISTS `bff_blog_posts_tags`;
DROP TABLE IF EXISTS `bff_blog_tags`;
DROP TABLE IF EXISTS `bff_blog_posts_lang`;
DROP TABLE IF EXISTS `bff_blog_posts`;

-- ------------------------------------------------------------------------------------
-- Посты
CREATE TABLE `bff_blog_posts` (
  `id` int(11) unsigned NOT NULL auto_increment,
  `cat_id` int(11) unsigned NOT NULL default 0,
  `link` text, -- Ссылка на топик
  `content` text, -- содержание
  `enabled` tinyint(1) unsigned NOT NULL default 0,
  `fav` int(11) unsigned NOT NULL default 0,
  `created` timestamp NOT NULL default '0000-00-00 00:00:00',
  `modified` timestamp NOT NULL default '0000-00-00 00:00:00',
  `user_id` int(11) unsigned NOT NULL default 0,
  `mtemplate` tinyint(1) unsigned NOT NULL default 1,
  PRIMARY KEY (id),
  KEY posts_cat (cat_id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;

-- ------------------------------------------------------------------------------------
-- Посты - мультиязычные данные
CREATE TABLE `bff_blog_posts_lang` (
  `id` int(11) unsigned NOT NULL,
  `lang` char(2) NOT NULL default '',
  `title` varchar(200) NOT NULL default '', -- заголовок
  `textshort` text,
  `mtitle` varchar(200) NOT NULL default '',
  `mkeywords` text,
  `mdescription` text,
  `content_search` text,
  UNIQUE KEY `lang` (`id`,`lang`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- ------------------------------------------------------------------------------------
-- Теги
CREATE TABLE `bff_blog_tags` (
  `id` int(11) unsigned NOT NULL auto_increment,
  `tag` varchar(255) NOT NULL,
  `moderated` tinyint(1) unsigned NOT NULL default 1,
  PRIMARY KEY (id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;

-- ------------------------------------------------------------------------------------
-- Связь постов с тегами
CREATE TABLE `bff_blog_posts_tags` (
  `post_id` int(11) unsigned NOT NULL,
  `tag_id` int(11) unsigned NOT NULL,
  KEY `post_id` (`post_id`),
  FOREIGN KEY (post_id) REFERENCES bff_blog_posts(id) ON DELETE CASCADE,
  FOREIGN KEY (tag_id) REFERENCES bff_blog_tags(id) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- ------------------------------------------------------------------------------------
-- Категории
CREATE TABLE `bff_blog_categories` (
  `id` int(11) unsigned NOT NULL auto_increment,
  `pid` int(11) unsigned default NULL,
  `keyword` varchar(150) NOT NULL default '',
  `enabled` tinyint(1) unsigned NOT NULL default 0,
  `created` timestamp NOT NULL default '0000-00-00 00:00:00',
  `modified` timestamp NOT NULL default '0000-00-00 00:00:00',
  `numleft` int(11) default NULL,
  `numright` int(11) default NULL,
  `numlevel` int(11) default NULL,
  `mtemplate` tinyint(1) unsigned NOT NULL default 1,
  PRIMARY KEY (id),
  UNIQUE KEY (id,numleft,numright)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;

-- ------------------------------------------------------------------------------------
-- Категории - мультиязычные данные
CREATE TABLE `bff_blog_categories_lang` (
  `id` int(11) unsigned NOT NULL,
  `lang` char(2) NOT NULL default '',
  `title` varchar(200) NOT NULL default '',
  `mtitle` varchar(200) NOT NULL default '',
  `mkeywords` text,
  `mdescription` text,
  UNIQUE KEY `lang` (`id`,`lang`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

INSERT INTO `bff_blog_categories` (id,pid,numleft,numright,numlevel) VALUES (1,0,1,2,0);
INSERT INTO `bff_blog_categories_lang` (id,title,lang) VALUES (1,'Корневой раздел','ru');
INSERT INTO `bff_blog_categories_lang` (id,title,lang) VALUES (1,'Корневой раздел','uk');

-- --------------------------------------------------------
-- права групп пользователей
DELETE FROM `bff_module_methods` WHERE module IN('blog');
INSERT INTO `bff_module_methods` (`module`, `method`, `title`, `number`) VALUES
('blog', 'blog', 'Блог', 90),
('blog', 'posts', 'Управление постами', 1),
('blog', 'categories', 'Управление категориями', 2),
('blog', 'tags', 'Управление тегами', 3),
('blog', 'seo', 'SEO', 4);
