
DROP TABLE IF EXISTS `bff_shops_svc_price`;
DROP TABLE IF EXISTS `bff_shops_claims`;
DROP TABLE IF EXISTS `bff_shops_in_categories`;
DROP TABLE IF EXISTS `bff_shops_in_categories_bbs`;
DROP TABLE IF EXISTS `bff_shops_categories_lang`;
DROP TABLE IF EXISTS `bff_shops_categories`;
DROP TABLE IF EXISTS `bff_shops`;

-- --------------------------------------------------------
-- Магазины
CREATE TABLE `bff_shops` (
  id int(11) unsigned NOT NULL auto_increment,
  id_ex character(6), -- расширитель id
  user_id int(11) unsigned NOT NULL default 0,
  link text, -- URL магазина
  keyword varchar(250) NOT NULL default '', -- URL keyword
  -- info
  title varchar(150) NOT NULL default '', -- проквоченное название магазина
  title_edit varchar(100) NOT NULL default '', -- название магазина указанное пользователем
  descr text, -- описание (чем занимается)
  phones text,
  phone varchar(50) NOT NULL default '',
  skype varchar(32) NOT NULL default '',
  icq varchar(20) NOT NULL default '',
  site varchar(200) NOT NULL default '', -- ссылка на сайт
  social text, -- соц. сети
  logo varchar(50) NOT NULL default '', -- логотип
  -- address
  reg1_country int(11) unsigned NOT NULL default 0,
  reg2_region int(11) unsigned NOT NULL default 0,
  reg3_city int(11) unsigned NOT NULL default 0,
  region_id int(11) unsigned NOT NULL default 0, -- регион деятельности
  addr_addr varchar(400) NOT NULL default '', -- точный адрес
  addr_lat float(10,6) NOT NULL default 0,
  addr_lon float(10,6) NOT NULL default 0,
  -- other
  created timestamp NOT NULL default '0000-00-00 00:00:00',
  items int unsigned NOT NULL DEFAULT 0,
  items_last timestamp NOT NULL default '0000-00-00 00:00:00',
  -- status
  status tinyint(1) unsigned NOT NULL default 1, -- статус
  status_prev tinyint(1) unsigned NOT NULL default 1, -- предыдущий статус
  status_changed timestamp NOT NULL default '0000-00-00 00:00:00', -- дата последнего изменения статуса
  moderated tinyint(1) unsigned NOT NULL default 0,
  blocked_reason text, -- причина блокировки
  claims_cnt smallint(5) unsigned NOT NULL default 0, -- счетчик жалоб на магазин
  -- услуги
  svc int(11) unsigned NOT NULL default 0, -- битовое поле активированных услуг
  svc_fixed_to timestamp NOT NULL default '0000-00-00 00:00:00', -- срок окончания действия услуги "Закрепление"
  svc_fixed_order timestamp NOT NULL default '0000-00-00 00:00:00', -- дата активации услуги "Закрепления" (для сортировки среди "Закрепленных")
  svc_marked_to timestamp NOT NULL default '0000-00-00 00:00:00', -- срок окончания действия услуги "Выделение"
  PRIMARY KEY (id),
  INDEX (user_id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;

-- --------------------------------------------------------
-- Категории магазинов
CREATE TABLE `bff_shops_categories` (
  id int(11) unsigned NOT NULL AUTO_INCREMENT,
  pid int(11) unsigned NOT NULL default 0,
  numleft int(11) unsigned default NULL,
  numright int(11) unsigned default NULL,
  numlevel int(11) unsigned default NULL,
  title varchar(100) NOT NULL default '',
  keyword varchar(300) NOT NULL default '',
  keyword_edit varchar(200) NOT NULL default '',
  icon_b varchar(20) NOT NULL default '',
  icon_s varchar(20) NOT NULL default '',
  enabled tinyint(1) unsigned NOT NULL default 1,
  created timestamp NOT NULL default '0000-00-00 00:00:00',
  modified timestamp NOT NULL default '0000-00-00 00:00:00',
  shops mediumint unsigned NOT NULL default 0,
  mtemplate tinyint(1) unsigned NOT NULL default 1,
  PRIMARY KEY (id),
  KEY keyword (keyword),
  UNIQUE KEY id (id,numleft,numright)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;

CREATE TABLE `bff_shops_categories_lang` (
  id int(11) unsigned NOT NULL,
  lang char(2) NOT NULL default '',
  title varchar(100) NOT NULL default '',
  mtitle text,
  mkeywords text,
  mdescription text,
  seotext text,
  titleh1 text,
  breadcrumb text,
  UNIQUE KEY lang (id,lang),
  FOREIGN KEY (id) REFERENCES bff_shops_categories(id) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- Связь магазинов с категориями
CREATE TABLE `bff_shops_in_categories` (
  id int(11) unsigned NOT NULL AUTO_INCREMENT,
  shop_id int(11) unsigned NOT NULL, -- id магазина
  category_id int(11) unsigned NOT NULL, -- id категории
  is_parent tinyint(1) unsigned NOT NULL default 0, -- ip parent категории
  num int(11) unsigned NOT NULL default 0, -- порядок связи магазина с категориями
  PRIMARY KEY (id),
  UNIQUE KEY shop_category (shop_id,category_id),
  FOREIGN KEY (shop_id) REFERENCES bff_shops(id) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;

INSERT INTO `bff_shops_categories` (id, pid, numleft, numright, numlevel, title) VALUES(1, 0, 1, 2, 0, 'Корневой раздел');
INSERT INTO `bff_shops_categories_lang` (id, lang, title) VALUES(1, 'ru', 'Корневой раздел');

-- --------------------------------------------------------
-- Категории магазинов (в связке с объявлениями)
CREATE TABLE `bff_shops_in_categories_bbs` (
  id int(11) unsigned NOT NULL AUTO_INCREMENT,
  shop_id int(11) unsigned NOT NULL,
  category_id int(11) unsigned NOT NULL,
  numlevel tinyint(1) unsigned NOT NULL default 0,
  items mediumint unsigned NOT NULL default 0,
  PRIMARY KEY (id),
  UNIQUE KEY shop_category (shop_id,category_id),
  FOREIGN KEY (shop_id) REFERENCES bff_shops(id) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;

-- ------------------------------------------------------------------------------------
-- Заявки на закрепление
DROP TABLE IF EXISTS `bff_shops_requests`;
CREATE TABLE `bff_shops_requests` (
  id int(11) unsigned NOT NULL auto_increment,
  user_id int(11) unsigned NOT NULL default 0, -- id пользователя или 0
  user_ip int(11) unsigned NOT NULL default 0,
  shop_id int(11) unsigned NOT NULL, -- id магазина
  name varchar(200) NOT NULL default '',
  phone varchar(50) NOT NULL default '',
  email varchar(100) NOT NULL default '',
  description text,
  created timestamp NOT NULL default '0000-00-00 00:00:00',
  modified timestamp NOT NULL default '0000-00-00 00:00:00',
  viewed tinyint(1) unsigned default 0,
  PRIMARY KEY (id),
  FOREIGN KEY (shop_id) REFERENCES bff_shops(id) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;

-- ------------------------------------------------------------------------------------
-- Жалобы на магазины
CREATE TABLE `bff_shops_claims` (
  id int(11) unsigned NOT NULL auto_increment,
  shop_id int(11) unsigned NOT NULL,
  user_id int(11) unsigned default 0,
  user_ip varchar(15) NOT NULL default '',
  reason int(11) unsigned default 0,
  message text,
  viewed tinyint(1) unsigned NOT NULL default 0,
  created timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY (id),
  FOREIGN KEY (shop_id) REFERENCES bff_shops(id) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;

-- ------------------------------------------------------------------------------------
-- Региональная стоимость услуг для магазинов
CREATE TABLE IF NOT EXISTS `bff_shops_svc_price` (
  id int(11) unsigned NOT NULL default 0,
  svc_id int(11) unsigned NOT NULL default 0,
  price float(10,0) NOT NULL default 0,
  region_id int(11) unsigned NOT NULL default 0,
  num int(11) unsigned NOT NULL default 0,
  INDEX (svc_id),
  INDEX (svc_id, region_id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- ------------------------------------------------------------------------------------
-- Услуги
DELETE FROM `bff_svc` WHERE id IN(64, 128);
INSERT INTO `bff_svc` (`id`, `type`, `keyword`, `module`, `module_title`, `title`, `price`, `settings`, `modified`, `modified_uid`, `icon_b`, `icon_s`, `num`) VALUES
(64, 1, 'fix_shop', 'shops', 'Магазины', 'Закрепление', 2, 'a:6:{s:6:"period";i:5;s:5:"color";s:0:"";s:2:"on";i:1;s:10:"title_view";a:2:{s:2:"ru";s:22:"Закрепление";s:2:"uk";s:22:"Закрепление";}s:11:"description";a:2:{s:2:"ru";s:225:"Закрепив магазин вы поднимаете его в начало списка магазинов на 5 дней.\r\n - в 10 раз больше просмотров\r\n- в 5 раз больше откликов";s:2:"uk";s:225:"Закрепив магазин вы поднимаете его в начало списка магазинов на 5 дней.\r\n - в 10 раз больше просмотров\r\n- в 5 раз больше откликов";}s:16:"description_full";a:2:{s:2:"ru";s:241:"<p>Закрепив магазин вы поднимаете его в начало списка магазинов на 5 дней.</p><p>- в 10 раз больше просмотров</p><p>- в 5 раз больше откликов</p>";s:2:"uk";s:241:"<p>Закрепив магазин вы поднимаете его в начало списка магазинов на 5 дней.</p><p>- в 10 раз больше просмотров</p><p>- в 5 раз больше откликов</p>";}}', '2014-07-12 18:22:10', 1, 'f532.png', '50d7.png', 8),
(128, 1, 'mark_shop', 'shops', 'Магазины', 'Выделение', 3, 'a:6:{s:6:"period";i:7;s:5:"color";s:0:"";s:2:"on";i:1;s:10:"title_view";a:2:{s:2:"ru";s:18:"Выделение";s:2:"uk";s:18:"Выделение";}s:11:"description";a:2:{s:2:"ru";s:272:"Магазин выделяется желтым фоном на период 7 дней. \r\nВыделение магазина — отличный способ сделать ваш магазин более ярким и заметным на фоне соседних.";s:2:"uk";s:272:"Магазин выделяется желтым фоном на период 7 дней. \r\nВыделение магазина — отличный способ сделать ваш магазин более ярким и заметным на фоне соседних.";}s:16:"description_full";a:2:{s:2:"ru";s:294:"<p>Магазин выделяется желтым фоном&nbsp;на период 7 дней.&nbsp;</p><p>Выделение магазина — отличный способ сделать ваш магазин более ярким и заметным на фоне соседних.</p>";s:2:"uk";s:294:"<p>Магазин выделяется желтым фоном&nbsp;на период 7 дней.&nbsp;</p><p>Выделение магазина — отличный способ сделать ваш магазин более ярким и заметным на фоне соседних.</p>";}}', '2014-06-23 11:24:41', 1, '3ae8.png', '976d.png', 9);

-- --------------------------------------------------------
-- Права групп пользователей
DELETE FROM `bff_module_methods` WHERE module IN('shops');
INSERT INTO `bff_module_methods` (`module`, `method`, `title`, `number`) VALUES
('shops', 'shops', 'Магазины', 50),
('shops', 'shops-listing', 'Просмотр списка магазинов', 1),
('shops', 'shops-requests', 'Управление списком запросов на открытие и закрепление', 2),
('shops', 'shops-edit', 'Управление магазинами (добавление/редактирование/удаление)', 3),
('shops', 'shops-moderate', 'Модерация объявлений (блокирование/одобрение)', 4),
('shops', 'claims-listing', 'Просмотр списка жалоб', 5),
('shops', 'claims-edit', 'Управление жалобами', 6),
('shops', 'categories', 'Управление категориями', 7),
('shops', 'svc', 'Управление услугами', 8),
('shops', 'seo', 'SEO', 9),
('shops', 'settings', 'Дополнительные настройки', 10);