SET FOREIGN_KEY_CHECKS=0;

-- --------------------------------------------------------
-- Users: пользователи

DROP TABLE IF EXISTS `bff_users`;
CREATE TABLE `bff_users` (
  user_id int(11) unsigned NOT NULL auto_increment,
  user_id_ex character(6), -- расширитель id
  admin tinyint(1) unsigned NOT NULL default 0,
  member tinyint(1) unsigned NOT NULL default 1,
  login varchar(50) NOT NULL default '',
  password varchar(32) NOT NULL default '',
  password_salt character(4),
  email varchar(100) NOT NULL default '',
  balance float(10,2) NOT NULL default 0,
  balance_spent float(10,2) NOT NULL default 0,
  region_id int(11) unsigned NOT NULL default 0,
  addr_addr varchar(300) NOT NULL default '', -- адрес
  addr_lat float(10,6) NOT NULL default 0, -- адрес, координата latitude
  addr_lon float(10,6) NOT NULL default 0, -- адрес, координата longitude
  shop_id int(11) unsigned NOT NULL default 0, -- id магазина (0 - магазин еще не создан)
  name varchar(200) NOT NULL default '',
  surname varchar(100) NOT NULL default '',
  avatar varchar(50) NOT NULL default '',
  avatar_crop text,
  created timestamp NOT NULL default '0000-00-00 00:00:00',
  created_ip int unsigned NOT NULL default 0,
  blocked smallint NOT NULL default 0,
  blocked_reason text,
  deleted tinyint(1) unsigned NOT NULL default 0,
  activate_key varchar(32) NOT NULL default '', -- ключ используемый для: активации аккаунта, восстановления пароля
  activate_expire timestamp NOT NULL default '0000-00-00 00:00:00',
  activated tinyint(1) unsigned NOT NULL default 0,
  subscribed tinyint(1) unsigned NOT NULL default 0,
  birthdate date NOT NULL default '1901-01-01',
  sex tinyint(1) unsigned NOT NULL default 0,
  about text,
  phones text,
  phone varchar(50) NOT NULL default '',
  skype varchar(32) NOT NULL default '',
  icq varchar(20) NOT NULL default '',
  site varchar(200) NOT NULL default '',
  im_noreply smallint NOT NULL default 0,
  enotify smallint NOT NULL default 0,
  social_id varchar(100) NOT NULL default '', -- id соц.аккаунта
  PRIMARY KEY (user_id),
  KEY (login)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 AUTO_INCREMENT=10;

-- --------------------------------------------------------
-- Пользователи (динамические данные: данные о последней активности, счетчики)
-- last_login - время последней авторизации
-- last_login2 - время предпоследней авторизации
-- last_login_ip - IP последней авторизации
-- last_activity - время последней активности
-- last_comment - время последнего комментария
-- items - кол-во объявлений
-- items_fav - кол-во избранных объявлений
-- internalmail_new - кол-во новых сообщений

DROP TABLE IF EXISTS `bff_users_stat`;
CREATE TABLE `bff_users_stat` (
  user_id int(11) unsigned NOT NULL,
  last_login timestamp NOT NULL default '0000-00-00 00:00:00',
  last_login2 timestamp NOT NULL default '0000-00-00 00:00:00',
  last_login_ip int unsigned NOT NULL default 0,
  last_activity timestamp NOT NULL default '0000-00-00 00:00:00',
  last_comment timestamp NOT NULL DEFAULT '0000-00-00 00:00:00',
  session_id varchar(50) NOT NULL default '',
  items smallint unsigned NOT NULL default 0,
  items_fav smallint unsigned NOT NULL default 0,
  internalmail_new smallint unsigned NOT NULL default 0
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

INSERT INTO `bff_users_stat` (user_id) VALUES (1);

-- --------------------------------------------------------
-- Users: социальные сети
-- provider_id - идентификатор соц. сети, UsersSocial::PROVIDER_, ...
-- profile_id - id профиля в соц. сети
-- profile_data - данные профиля соц. сети, полученные при авторизации

DROP TABLE IF EXISTS `bff_users_social`;
CREATE TABLE `bff_users_social` (
  id int(11) unsigned NOT NULL auto_increment,
  user_id int(11) unsigned NOT NULL default 0,
  provider_id int(11) unsigned NOT NULL default 0,
  profile_id varchar(100) NOT NULL default '',
  profile_data text,
  PRIMARY KEY (id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;

-- --------------------------------------------------------
-- Users: группы

DROP TABLE IF EXISTS `bff_users_groups`;
CREATE TABLE `bff_users_groups` (
  group_id smallint(5) unsigned NOT NULL auto_increment,
  issystem tinyint(1) unsigned NOT NULL default 0,
  adminpanel tinyint(1) unsigned NOT NULL default 0,
  keyword varchar(30) NOT NULL default '',
  title character(100) NOT NULL default '',
  color character(40) NOT NULL default '#000',
  created timestamp NOT NULL default '0000-00-00 00:00:00',
  modified timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY (group_id),
  UNIQUE KEY groups_key (group_id, keyword)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 AUTO_INCREMENT=30 ;

INSERT INTO `bff_users_groups` (group_id, issystem, adminpanel, keyword, title, color, created, modified) VALUES
(7, 1, 1, 'x71', 'Главный администратор', '#55AF0A', '0000-00-00 00:00:00', '0000-00-00 00:00:00'),
(17, 1, 1, 'c60', 'Модератор', '#D8062D', '0000-00-00 00:00:00', '2010-05-12 13:34:24'),
(22, 1, 0, 'z24', 'Пользователь', 'gray', '0000-00-00 00:00:00', '2009-04-06 01:29:00');

-- --------------------------------------------------------
-- Users: группы, права доступа

DROP TABLE IF EXISTS `bff_users_groups_permissions`;
CREATE TABLE `bff_users_groups_permissions` (
  unit_id int(11) unsigned NOT NULL default 0,
  unit_type character(60) NOT NULL default '',
  item_id int(11) unsigned NOT NULL default 0,
  item_type character(60) NOT NULL default '',
  created timestamp NOT NULL default '0000-00-00 00:00:00'
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

-- --------------------------------------------------------
-- Users: пользователи в группах

DROP TABLE IF EXISTS `bff_user_in_group`;
CREATE TABLE `bff_user_in_group` (
  user_id int(11) unsigned NOT NULL default 0,
  group_id smallint(5) unsigned NOT NULL default 0,
  created timestamp NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY (user_id, group_id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;


-- --------------------------------------------------------
-- Администратор

INSERT INTO `bff_users` (`user_id`, `user_id_ex`, `admin`, `member`, `login`, `password`, `password_salt`, `email`, `balance`, `balance_spent`, `region_id`, `addr_addr`, `addr_lat`, `addr_lon`, `shop_id`, `name`, `surname`, `avatar`, `avatar_crop`, `created`, `created_ip`, `blocked`, `blocked_reason`, `deleted`, `activate_key`, `activate_expire`, `activated`, `subscribed`, `birthdate`, `sex`, `about`, `phones`, `phone`, `skype`, `icq`, `site`, `im_noreply`, `enotify`, `social_id`) VALUES
(1, '790a3d', 1, 1, 'admin', 'b99818a65d4da090e230e0cc54c1f9af', '9Z2O', 'admin@example.com', 0.00, 0.00, 255, '', 50.524200, 30.797100, 0, 'Администрация', '', 'd541c7.png', NULL, '2012-01-01 18:12:01', 0, 0, '', 0, '', '2012-01-01 18:12:02', 1, 1, '1985-01-01', 2, '', 'a:0:{}', '', '', '', '', 0, 0, '');


INSERT INTO `bff_user_in_group` (`user_id`, `group_id`, `created`) VALUES
(1, 7, '2013-07-12 18:42:54'),
(1, 22, '2013-07-12 18:42:54');

INSERT INTO `bff_users_stat` (`user_id`, `last_login`, `last_login2`, `last_login_ip`, `last_activity`, `last_comment`, `session_id`, `items`, `items_fav`, `internalmail_new`) VALUES
(1, '2013-07-12 20:21:07', '2013-07-12 18:42:56', 2130706433, '2013-07-12 20:55:49', '0000-00-00 00:00:00', '', 0, 0, 0);

-- --------------------------------------------------------
-- права групп пользователей
DELETE FROM `bff_module_methods` WHERE module IN('users');
INSERT INTO `bff_module_methods` (`module`, `method`, `title`, `number`) VALUES
('users', 'users', 'Пользователи', 60),
('users', 'profile', 'Ваш профиль', 1),
('users', 'members-listing', 'Список пользователей', 2),
('users', 'admins-listing', 'Список администраторов', 3),
('users', 'users-edit', 'Управление пользователями', 4),
('users', 'ban', 'Блокировка пользователей', 5),
('users', 'seo', 'SEO', 6),
('users', 'groups-listing', 'Список групп пользователей', 7),
('users', 'groups-edit', 'Управление группами пользователей', 8);

SET FOREIGN_KEY_CHECKS=1;