/**
 * Класс для работы с точкой на карте Yandex
 * @author Tamaranga | tamaranga.com
 * @version 0.37
 * @modified 14.jan.2014
 * @return {Object}
 */

function bffmapEditorYandex() {

    var o, map, $coords = [], marker, markerIsDraggable;
    
    function init(options, center)
    {
        setOptions(options);

        // центрируем, исходя из указанных координат
        if (center) {
            var coords = [];
            if($coords.length == 1) {
                coords = $coords.val().split(',');
            } else if($coords.length == 2) {
                coords = [$coords[0].val(), $coords[1].val()];
            }
            map.setCenter(coords);
        }

        // создаем маркер
        marker = new ymaps.Placemark( map.getCenter(), o.markerProperties, o.markerOptions );
        map.geoObjects.add( marker );
        if( markerIsDraggable ) {
             // заверешение перетаскивания маркера
             marker.events.add('dragend', function(e) {
                updateMarker( marker.geometry.getCoordinates() );
             });
        }

        // обновляем позицию маркера по клику
        map.events.add('click', function (e) {
            updateMarker( e.get('coordPosition') );
        });

        // SearchControl
        if(o.searchInit!==false) {
            var oMapSearchCtrl = new ymaps.control.SearchControl(o.searchOptions);
            oMapSearchCtrl.events.add('resultselect', function(e) {
                oMapSearchCtrl.getResult(e.get('resultIndex')).then(function (res) { // GeoObject
                    var coords = res.geometry.getCoordinates();
                    updateMarker( coords, false );
                    map.panTo( coords );
                    updateAddress( res.properties.get('name') );
                });
            });
            map.controls.add(oMapSearchCtrl, {left:'40px', top:'10px'});
        }
    }

    function setOptions(options)
    {
        o = $.extend({
            map: false, //element (ymaps.Map)
            coords: [], //element (для хранения координат): $latlong, [$lat,$long], []
            address: false, //element (для хранения адреса)
            addressKind: 'normal', // 'house','street','metro','district','locality' или false
            markerProperties: {}, // свойства маркера
            markerOptions: {draggable: true, hasBalloon: false}, // опции маркера
            searchInit: false, // инициализировать SearchControl
            searchOptions: {}, // настройки SearchControl
            updateAddressIgnoreClass: '' // изменять строку адреса если не указан класс
        }, options || {});

        if( o.searchInit!==false ) {
            o.searchOptions = $.extend({
                width: 500,
                noPlacemark: true, noCentering:true, noPopup: true,
                resultsPerPage: 3}, o.searchOptions);
        }

        if(options.hasOwnProperty('map')) map = o.map;
        if(options.hasOwnProperty('coords')) $coords = o.coords;

        markerIsDraggable = ( o.markerOptions['draggable'] === true );
    }

    function updateMarker(coords, updateAddr, panToAddr)
    {
        marker.geometry.setCoordinates( coords );

        if( $coords.length == 1 ) {
            $coords.val( [ coords[0].toPrecision(6),
                           coords[1].toPrecision(6)
                           ].join(',') );
        } else if($coords.length == 2) {
            $coords[0].val( coords[0].toPrecision(6) );
            $coords[1].val( coords[1].toPrecision(6) );
        }

        if( updateAddr!==false ) {
            updateAddress( coords );
        }
        if( (panToAddr || panToAddr === 0) && $(map.container.getElement()).parent().is(':visible')) {
            map.panTo(coords, {duration: panToAddr});
        }

        if(o.onUpdate) o.onUpdate(marker);
    }
    
    function updateAddress(a)
    {
        if( o.address === false ||
            ( o.updateAddressIgnoreClass.length > 0 && o.address.hasClass(o.updateAddressIgnoreClass) ) )
            return;

        try
        {
            if( $.isArray(a) ) // координаты
            {
                var coords = a;
                var opts = {};
                var _addr = '';

                switch(o.addressKind) {
                    case false: // full address
                    {
                        ymaps.geocode(coords, opts).then(function (res) {
                            var names = [];
                            res.geoObjects.each(function(obj) {
                                names.push( obj.properties.get('name') );
                            });
                            _addr = names.reverse().join(', ');
                            o.address.val( _addr );
                        });
                    } break;
                    case 'normal': { // city + street + house
                        opts['kind']  = 'house';
                        ymaps.geocode(coords, opts).then(function (res) {
                            _addr = res.geoObjects.get(0).properties.get('name');
                            coords = res.geoObjects.get(0).geometry.getCoordinates();
                            ymaps.geocode(coords, {kind: 'locality'}).then(function (res) {
                                try{
                                    _addr = res.geoObjects.get(0).properties.get('name')+', '+_addr;
                                    o.address.val( _addr );
                                } catch(e) {}
                            });
                        });
                    } break;
                    default: { // kind = 'house','street','metro','district','locality'
                        opts['kind']  = o.addressKind;
                        ymaps.geocode(coords, opts).then(function (res) {
                            var names = [];
                            res.geoObjects.each(function(obj) {
                                names.push(obj.properties.get('name'));
                            });
                            try{
                                _addr = res.geoObjects.get(0).properties.get('name');
                                o.address.val( _addr );
                            } catch(e) {}
                        });
                    } break;
                }
            } else { // строка
                o.address.val( new String(a) );
            }

        } catch(e){
            
        }
    }
    
    function fireUpdate()
    {
        updateMarker( marker.geometry.getCoordinates() );
    }

    function centerByMarker()
    {
        map.setCenter( marker.geometry.getCoordinates() );
    }
    
    return {
        init: init,
        setOptions: setOptions,
        fireUpdate: fireUpdate,
        updateMarker: updateMarker,
        centerByMarker: centerByMarker,
        search: function(addr, panDuration, callback)
        {
            var geo = ymaps.geocode(addr, {results: 1});
            geo.then(function(res){
                if(res.geoObjects.getLength()) {
                    var obj = res.geoObjects.get(0);
                    var coords = obj.geometry.getCoordinates();
                    updateMarker(coords, false, panDuration);
                    if(callback) callback(obj, coords);
                }
            });
        }
    };
}